from ortools.sat.python import cp_model
import traceback

class NursesPartialSolutionPrinter(cp_model.CpSolverSolutionCallback):
    """Print intermediate solutions."""

    def __init__(self, shifts, num_nurses, num_days, num_shifts, shift_requests, sols, output_file):
        cp_model.CpSolverSolutionCallback.__init__(self)
        self._shifts = shifts
        self._num_nurses = num_nurses
        self._num_days = num_days
        self._num_shifts = num_shifts
        self._shift_requests = shift_requests
        self._solutions = set(sols)
        self._solution_count = 0
        self._output_file = output_file

    def on_solution_callback(self) :
        try:
            print('Solution', self._solution_count)
            self._output_file.write('Solution {}\n'.format(self._solution_count) )
            for d in range(self._num_days):
                print('Day', d)
                self._output_file.write('Day {}\n'.format(d) )
                for n in range(self._num_nurses):
                    for s in range(self._num_shifts):
                        if self.Value(self._shifts[(n, d, s)]) == 1:
                            if self._shift_requests[n][d][s] == 1:
                                print('Nurse', n, 'works shift', s, '(requested).')
                                self._output_file.write('Nurse {} works shift {} (requested).\n'.format(n,s))
                            else:
                                print('Nurse', n, 'works shift', s, '(not requested).')
                                self._output_file.write('Nurse {} works shift {} (not requested).\n'.format(n,s))
                print()
                self._output_file.write('\n')
            self._solution_count += 1

        except Exception as e:
            print("i caught an error")
            traceback.print_exc()
            raise e

    def solution_count(self):
        return self._solution_count