from ortools.linear_solver import pywraplp

from data import create_data

def main():
    data = create_data()
    output_file = open('MILP-problem-solution-printer.txt','a')
    # Create the mip solver with the SCIP backend.
    solver = pywraplp.Solver.CreateSolver('SCIP')

    infinity = solver.infinity()
    x = {}
    for j in range(data['num_vars']):
        x[j] = solver.IntVar(0, infinity, 'x[%i]' % j)
    print('Number of variables =', solver.NumVariables())
    output_file.write('Number of variables = {}\n'.format(solver.NumVariables()))

    for i in range(data['num_constraints']):
        constraint = solver.RowConstraint(0, data['bounds'][i], '')
        for j in range(data['num_vars']):
            constraint.SetCoefficient(x[j], data['constraint_coeffs'][i][j])
    print('Number of constraints = ', solver.NumConstraints())
    output_file.write('Number of constraints ={}\n'.format(solver.NumConstraints()))
    # In Python, you can also set the constraints as follows.
    # for i in range(data['num_constraints']):
    #  constraint_expr = \
    # [data['constraint_coeffs'][i][j] * x[j] for j in range(data['num_vars'])]
    #  solver.Add(sum(constraint_expr) <= data['bounds'][i])

    objective = solver.Objective()
    for j in range(data['num_vars']):
        objective.SetCoefficient(x[j], data['obj_coeffs'][j])
    objective.SetMaximization()
    # In Python, you can also set the objective as follows.
    # obj_expr = [data['obj_coeffs'][j] * x[j] for j in range(data['num_vars'])]
    # solver.Maximize(solver.Sum(obj_expr))

    status = solver.Solve()

    if status == pywraplp.Solver.OPTIMAL:
        print('Objective value =', solver.Objective().Value())
        output_file.write('Objective value = {}\n'.format(solver.Objective().Value()))
        for j in range(data['num_vars']):
            print(x[j].name(), ' = ', x[j].solution_value())
            output_file.write('{} = {}\n'.format(x[j].name(), x[j].solution_value()))
        print()
        print('Problem solved in %f milliseconds' % solver.wall_time())
        print('Problem solved in %d iterations' % solver.iterations())
        print('Problem solved in %d branch-and-bound nodes' % solver.nodes())

        output_file.write('\n')
        output_file.write('Problem solved in %f milliseconds\n' % solver.wall_time())
        output_file.write('Problem solved in %d iterations\n' % solver.iterations())
        output_file.write('Problem solved in %d branch-and-bound nodes\n' % solver.nodes())
    else:
        print('The problem does not have an optimal solution.')
        output_file.write('The problem does not have an optimal solution.\n')
    
    output_file.close()


if __name__ == '__main__':
    main()
 